require 'securerandom'

class Cornflake
  module MacAddress
    RE = %r/(?:[^:\-]|\A)(?:[0-9A-F][0-9A-F][:\-]){5}[0-9A-F][0-9A-F](?:[^:\-]|\Z)/io

    def self.find
      cmds = '/sbin/ifconfig', '/bin/ifconfig', 'ifconfig', 'cat /sys/class/net/*/address'

      cmds.each do |cmd|
        mac = parse_mac_output(`#{cmd}`)
        return mac if mac
      end

      # Not found; randomize a mac address
      SecureRandom.hex(6).scan(/.{2}/).join(':')
    end

    private
    def self.parse_mac_output(output)
      output.split(/\n/).each do |line|
        mac = line[RE]
        return mac.strip if mac
      end
    end
  end
end
