#include <ngx_core.h>
#include <ngx_http.h>
#include <nginx.h>
#include <ngx_http_variables.h>
#include "cornflake.h"

static struct cornflake *g_cornflake;

ngx_int_t
ngx_cornflake_get_var(ngx_http_request_t *r,
	ngx_http_variable_value_t *v, uintptr_t data)
{
	char *uid = ngx_pnalloc(r->pool, 33);

	if (uid == NULL)
		return NGX_ERROR;

	if (cornflake_gen_hex(uid, 33, g_cornflake) < 0) {
		ngx_pfree(r->pool, uid);
		return NGX_ERROR;
	}

	v->len = 32;
	v->valid = 1;
	v->no_cacheable = 0;
	v->not_found = 0;
	v->data = (u_char *)uid;

	return NGX_OK;
}

static ngx_int_t add_variables(ngx_conf_t *cf)
{
	static ngx_str_t var_name = ngx_string("request_id");
	ngx_http_variable_t *var = NULL;

	g_cornflake = cornflake_new("/etc/cornflake");
	if (g_cornflake == NULL)
		return NGX_ERROR;

	var = ngx_http_add_variable(cf, &var_name, NGX_HTTP_VAR_NOHASH);
	if (var == NULL)
		return NGX_ERROR;

	var->get_handler = ngx_cornflake_get_var;
	return NGX_OK;
}

static ngx_http_module_t  cornflake_context = {
	add_variables,  /* preconfiguration */
	NULL,			/* postconfiguration */

	NULL,        /* create main configuration */
	NULL,        /* init main configuration */

	NULL,        /* create server configuration */
	NULL,        /* merge server configuration */

	NULL,        /* create location configuration */
	NULL         /* merge location configuration */
};

static ngx_command_t cornflake_commands[] = {
	ngx_null_command
};

ngx_module_t ngx_cornflake_module = {
	NGX_MODULE_V1,
	&cornflake_context,		/* module context */
	cornflake_commands,	/* module directives */
	NGX_HTTP_MODULE,		/* module type */
	NULL,                              /* init master */
	NULL,                              /* init module */
	NULL,                              /* init process */
	NULL,                              /* init thread */
	NULL,                              /* exit thread */
	NULL,                              /* exit process */
	NULL,                              /* exit master */
	NGX_MODULE_V1_PADDING
};
